'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function topPosition(domElt) {
    if (!domElt) {
        return 0;
    }

    return domElt.offsetTop + topPosition(domElt.offsetParent);
}

const DefaultLoader = _react2.default.createElement(
    'div',
    { className: 'loader' },
    'Loading ...'
);

const InfiniteScroll = _react2.default.createClass({
    displayName: 'InfiniteScroll',


    propTypes: {
        hasMore: _react2.default.PropTypes.bool.isRequired,
        currentPage: _react2.default.PropTypes.number.isRequired,
        threshold: _react2.default.PropTypes.number,
        loadMore: _react2.default.PropTypes.func.isRequired,
        loader: _react2.default.PropTypes.element
    },

    getDefaultProps() {
        return {
            hasMore: false,
            loadMore() {},

            threshold: 1500
        };
    },

    componentDidMount() {
        this.attachScrollListener();
    },

    componentDidUpdate() {
        this.attachScrollListener();
    },

    componentWillUnmount() {
        this.detachScrollListener();
    },

    scrollListener() {
        if (!this.props.hasMore) {
            this.detachScrollListener();

            return;
        }

        const el = _reactDom2.default.findDOMNode(this);
        const scrollTop = window.pageYOffset !== undefined ? window.pageYOffset : (document.documentElement || document.body.parentNode || document.body).scrollTop;
        if (topPosition(el) + el.offsetHeight - scrollTop - window.innerHeight < Number(this.props.threshold)) {
            this.detachScrollListener();

            // call loadMore after detachScrollListener to allow
            // for non-async loadMore functions
            this.props.loadMore(this.props.currentPage + 1);
        }
    },

    attachScrollListener() {
        if (!this.props.hasMore) {
            return;
        }

        window.addEventListener('scroll', this.scrollListener);
        window.addEventListener('resize', this.scrollListener);
    },

    detachScrollListener() {
        window.removeEventListener('scroll', this.scrollListener);
        window.removeEventListener('resize', this.scrollListener);
    },

    render() {
        const props = this.props;

        return _react2.default.createElement(
            'div',
            { className: this.props.className },
            props.children,
            props.hasMore ? props.loader || DefaultLoader : null
        );
    }
});

exports.default = InfiniteScroll;